@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2019  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2021  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -283,7 +283,13 @@ static int d_avail(int d)
 	FD_SET(d, &rfds);
 	tv.tv_sec = 0;
 	tv.tv_usec = 0;
-	return select(d+1, &rfds, NULL, NULL, &tv);
+
+	for (;;) {
+		// Select may return -1 if interrupted.
+		int r = select(d+1, &rfds, NULL, NULL, &tv);
+		if (r >= 0)
+			return r;
+	}
 }
 
 
