@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2008  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2007-2009  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -23,9 +23,6 @@
  *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  *  SUCH DAMAGE.
- *
- *
- *  $Id: generate_m88k_bcnd.c,v 1.3.2.1 2008/01/18 19:12:27 debug Exp $
  */
 
 #include <stdio.h>
@@ -46,6 +43,8 @@ void print_function_name(int samepage, int n_bit, int m5)
 	case 0x1: printf("gt0"); break;
 	case 0x2: printf("eq0"); break;
 	case 0x3: printf("ge0"); break;
+	case 0x7: printf("not_maxneg"); break;
+	case 0x8: printf("maxneg"); break;
 	case 0xc: printf("lt0"); break;
 	case 0xd: printf("ne0"); break;
 	case 0xe: printf("le0"); break;
@@ -56,12 +55,14 @@ void print_function_name(int samepage, int n_bit, int m5)
 void print_operator(int m5)
 {
 	switch (m5) {
-	case 0x1: printf(">"); break;
-	case 0x2: printf("=="); break;
-	case 0x3: printf(">="); break;
-	case 0xc: printf("<"); break;
-	case 0xd: printf("!="); break;
-	case 0xe: printf("<="); break;
+	case 0x1: printf("> 0"); break;
+	case 0x2: printf("== 0"); break;
+	case 0x3: printf(">= 0"); break;
+	case 0x7: printf("!= 0x80000000UL"); break;
+	case 0x8: printf("== 0x80000000UL"); break;
+	case 0xc: printf("< 0"); break;
+	case 0xd: printf("!= 0"); break;
+	case 0xe: printf("<= 0"); break;
 	}
 }
 
@@ -77,9 +78,10 @@ void bcnd(int samepage, int n_bit, int m5)
 
 	/*  Easiest case is without the n_bit:  */
 	if (!n_bit) {
-		printf("\tif ((int32_t)reg(ic->arg[0]) ");
+		printf("\tif ((%sint32_t)reg(ic->arg[0]) ",
+		    (m5 == 7 || m5 == 8)? "u" : "");
 		print_operator(m5);
-		printf(" 0) {\n");
+		printf(") {\n");
 
 		if (samepage)
 			printf("\t\tcpu->cd.m88k.next_ic = (struct m88k_"
@@ -92,14 +94,20 @@ void bcnd(int samepage, int n_bit, int m5)
 		printf("\t}\n");
 	} else {
 		/*  n_bit, i.e. delay slot:  */
-		printf("\tint cond = (int32_t)reg(ic->arg[0]) ");
+		printf("\tint cond = (%sint32_t)reg(ic->arg[0]) ",
+		    (m5 == 7 || m5 == 8)? "u" : "");
 		print_operator(m5);
-		printf(" 0;\n");
+		printf(";\n");
 
-		printf("\tcpu->cd.m88k.delay_target = (cpu->pc\n");
-		printf("\t\t& ~((M88K_IC_ENTRIES_PER_PAGE-1)"
+		printf("\tSYNCH_PC;\n");
+
+		printf("\tif (cond)\n");
+		printf("\t\tcpu->cd.m88k.delay_target = (cpu->pc\n");
+		printf("\t\t\t& ~((M88K_IC_ENTRIES_PER_PAGE-1)"
 		    " << M88K_INSTR_ALIGNMENT_SHIFT))\n");
-		printf("\t\t+ ic->arg[2];\n");
+		printf("\t\t\t+ ic->arg[2];\n");
+		printf("\telse\n");
+		printf("\t\tcpu->cd.m88k.delay_target = cpu->pc + 8;\n");
 
 		printf("\tcpu->delay_slot = TO_BE_DELAYED;\n");
 		printf("\tic[1].f(cpu, ic+1);\n");
@@ -133,7 +141,7 @@ int main(int argc, char *argv[])
 	for (samepage=0; samepage<=1; samepage++)
 		for (n_bit=0; n_bit<=1; n_bit++)
 			for (m5=0; m5<=31; m5++) {
-				if (m5 == 1 || m5 == 2 || m5 == 3 ||
+				if (m5 == 1 || m5 == 2 || m5 == 3 || m5 == 7 || m5 == 8 ||
 				    m5 == 0xc || m5 == 0xd || m5 == 0xe)
 					bcnd(samepage, n_bit, m5);
 			}
@@ -147,7 +155,7 @@ int main(int argc, char *argv[])
 				if (m5 || n_bit || samepage)
 					printf(",\n");
 
-				if (m5 == 1 || m5 == 2 || m5 == 3 ||
+				if (m5 == 1 || m5 == 2 || m5 == 3 || m5 == 7 || m5 == 8 ||
 				    m5 == 0xc || m5 == 0xd || m5 == 0xe) {
 					if (samepage && n_bit)
 						printf("NULL");
