@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003-2009  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2003-2021  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -72,9 +72,10 @@
 
 extern int quiet_mode;
 
-#define	LE_MODE_LOOP		4
-#define	LE_MODE_DTX		2
-#define	LE_MODE_DRX		1
+#define	LE_MODE_PROM		0x8000
+#define	LE_MODE_LOOP		0x0004
+#define	LE_MODE_DTX		0x0002
+#define	LE_MODE_DRX		0x0001
 
 
 #define	N_REGISTERS		4
@@ -83,6 +84,8 @@ extern int quiet_mode;
 
 
 struct le_data {
+	struct nic_data	nic;
+
 	struct interrupt irq;
 	int		irq_asserted;
 
@@ -95,19 +98,22 @@ struct le_data {
 	int		reg_select;
 	uint16_t	reg[N_REGISTERS];
 
+	bool		bigendian;
+
 	unsigned char	*sram;
 
 	/*  Initialization block:  */
 	uint32_t	init_block_addr;
 
 	uint16_t	mode;
-	uint64_t	padr;	/*  MAC address  */
-	uint64_t	ladrf;
+	uint16_t	ladrf[4];
 	uint32_t	rdra;	/*  receive descriptor ring address  */
 	int		rlen;	/*  nr of rx descriptors  */
 	uint32_t	tdra;	/*  transmit descriptor ring address  */
 	int		tlen;	/*  nr ot tx descriptors  */
 
+	int		allmulti;/* receive all multicast packets */
+
 	/*  Current rx and tx descriptor indices:  */
 	int		rxp;
 	int		txp;
@@ -129,10 +135,12 @@ struct le_data {
  */
 static uint64_t le_read_16bit(struct le_data *d, int addr)
 {
-	/*  TODO: This is for little endian only  */
-	int x = d->sram[addr & (SRAM_SIZE-1)] +
+	if (d->bigendian)
+		return (d->sram[addr & (SRAM_SIZE-1)] << 8) +
+			(d->sram[(addr+1) & (SRAM_SIZE-1)]);
+
+	return d->sram[addr & (SRAM_SIZE-1)] +
 	       (d->sram[(addr+1) & (SRAM_SIZE-1)] << 8);
-	return x;
 }
 
 
@@ -143,9 +151,17 @@ static uint64_t le_read_16bit(struct le_data *d, int addr)
  */
 static void le_write_16bit(struct le_data *d, int addr, uint16_t x)
 {
-	/*  TODO: This is for little endian only  */
-	d->sram[addr & (SRAM_SIZE-1)] = x & 0xff;
-	d->sram[(addr+1) & (SRAM_SIZE-1)] = (x >> 8) & 0xff;
+	if (d->bigendian)
+	{
+		d->sram[addr & (SRAM_SIZE-1)] = (x >> 8) & 0xff;
+		d->sram[(addr+1) & (SRAM_SIZE-1)] = x & 0xff;
+	}
+	else
+	{
+		d->sram[addr & (SRAM_SIZE-1)] = x & 0xff;
+		d->sram[(addr+1) & (SRAM_SIZE-1)] = (x >> 8) & 0xff;
+	}
+
 }
 
 
@@ -157,6 +173,9 @@ static void le_write_16bit(struct le_data *d, int addr, uint16_t x)
  */
 static void le_chip_init(struct le_data *d)
 {
+	uint16_t tmp;
+	uint8_t macaddr[6];
+
 	d->init_block_addr = (d->reg[1] & 0xffff) + ((d->reg[2] & 0xff) << 16);
 	if (d->init_block_addr & 1)
 		fatal("[ le: WARNING! initialization block address "
@@ -165,13 +184,36 @@ static void le_chip_init(struct le_data *d)
 	debug("[ le: d->init_block_addr = 0x%06x ]\n", d->init_block_addr);
 
 	d->mode = le_read_16bit(d, d->init_block_addr + 0);
-	d->padr = le_read_16bit(d, d->init_block_addr + 2);
-	d->padr += (le_read_16bit(d, d->init_block_addr + 4) << 16);
-	d->padr += (le_read_16bit(d, d->init_block_addr + 6) << 32);
-	d->ladrf = le_read_16bit(d, d->init_block_addr + 8);
-	d->ladrf += (le_read_16bit(d, d->init_block_addr + 10) << 16);
-	d->ladrf += (le_read_16bit(d, d->init_block_addr + 12) << 32);
-	d->ladrf += (le_read_16bit(d, d->init_block_addr + 14) << 48);
+
+	/*
+	 * The MAC address is packed into the PADR field as 3 little-endian
+	 * 16-bit words.
+	 */
+	tmp = le_read_16bit(d, d->init_block_addr + 2);
+	macaddr[0] = (uint8_t)(tmp);
+	macaddr[1] = (uint8_t)(tmp >> 8);
+	tmp = le_read_16bit(d, d->init_block_addr + 4);
+	macaddr[2] = (uint8_t)(tmp);
+	macaddr[3] = (uint8_t)(tmp >> 8);
+	tmp = le_read_16bit(d, d->init_block_addr + 6);
+	macaddr[4] = (uint8_t)(tmp);
+	macaddr[5] = (uint8_t)(tmp >> 8);
+	memcpy(d->nic.mac_address, macaddr, sizeof(d->nic.mac_address));
+
+	/*
+	 * The muticast address filter is packed into the LADRF field
+	 * as 4 little-endian 16-bit words.
+	 */
+	d->ladrf[0] = le_read_16bit(d, d->init_block_addr + 8);
+	d->ladrf[1] = le_read_16bit(d, d->init_block_addr + 10);
+	d->ladrf[2] = le_read_16bit(d, d->init_block_addr + 12);
+	d->ladrf[3] = le_read_16bit(d, d->init_block_addr + 14);
+	if (d->ladrf[0] == 0xffff && d->ladrf[1] == 0xffff &&
+	    d->ladrf[2] == 0xffff && d->ladrf[3] == 0xffff)
+		d->allmulti = 1;
+	else
+		d->allmulti = 0;
+
 	d->rdra = le_read_16bit(d, d->init_block_addr + 16);
 	d->rdra += ((le_read_16bit(d, d->init_block_addr + 18) & 0xff) << 16);
 	d->rlen = 1 << ((le_read_16bit(d, d->init_block_addr + 18) >> 13) & 7);
@@ -179,13 +221,16 @@ static void le_chip_init(struct le_data *d)
 	d->tdra += ((le_read_16bit(d, d->init_block_addr + 22) & 0xff) << 16);
 	d->tlen = 1 << ((le_read_16bit(d, d->init_block_addr + 22) >> 13) & 7);
 
-	debug("[ le: DEBUG: mode              %04x ]\n", d->mode);
-	debug("[ le: DEBUG: padr  %016llx ]\n", (long long)d->padr);
-	debug("[ le: DEBUG: ladrf %016llx ]\n", (long long)d->ladrf);
-	debug("[ le: DEBUG: rdra            %06llx ]\n", d->rdra);
-	debug("[ le: DEBUG: rlen               %3i ]\n", d->rlen);
-	debug("[ le: DEBUG: tdra            %06llx ]\n", d->tdra);
-	debug("[ le: DEBUG: tlen               %3i ]\n", d->tlen);
+	debug("[ le: DEBUG: mode                 %04x ]\n", d->mode);
+	debug("[ le: DEBUG: padr    %02x:%02x:%02x:%02x:%02x:%02x ]\n",
+	      macaddr[0], macaddr[1], macaddr[2],
+	      macaddr[3], macaddr[4], macaddr[5]);
+	debug("[ le: DEBUG: ladrf %04x:%04x:%04x:%04x ]\n",
+	      d->ladrf[0], d->ladrf[1], d->ladrf[2], d->ladrf[3]);
+	debug("[ le: DEBUG: rdra               %06llx ]\n", d->rdra);
+	debug("[ le: DEBUG: rlen                  %3i ]\n", d->rlen);
+	debug("[ le: DEBUG: tdra               %06llx ]\n", d->tdra);
+	debug("[ le: DEBUG: tlen                  %3i ]\n", d->tlen);
 
 	/*  Set TXON and RXON, unless they are disabled by 'mode':  */
 	if (d->mode & LE_MODE_DTX)
@@ -198,6 +243,9 @@ static void le_chip_init(struct le_data *d)
 	else
 		d->reg[0] |= LE_RXON;
 
+	/*  Initialize promiscuous mode.  */
+	d->nic.promiscuous_mode = (d->mode & LE_MODE_PROM) ? 1 : 0;
+
 	/*  Go to the start of the descriptor rings:  */
 	d->rxp = d->txp = 0;
 
@@ -308,7 +356,8 @@ static void le_tx(struct net *net, struct le_data *d)
 		 *  the packet.
 		 */
 		if (enp) {
-			net_ethernet_tx(net, d, d->tx_packet, d->tx_packet_len);
+			net_ethernet_tx(net, &d->nic, d->tx_packet,
+			    d->tx_packet_len);
 
 			free(d->tx_packet);
 			d->tx_packet = NULL;
@@ -445,6 +494,62 @@ static void le_rx(struct net *net, struct le_data *d)
 }
 
 
+/*
+ *  le_rx_drop_packet():
+ *
+ *  Implement the logic to determine if we should drop a packet
+ *  before passing it to the guest.  Returns 1 if the packet was
+ *  dropped.
+ */
+static int
+le_rx_drop_packet(struct net *net, struct le_data *d)
+{
+	/* Only implement filtering if using a tap device. */
+	if (net->tapdev == NULL)
+		return 0;
+
+	/*
+	 * The network layer has already checked for our MAC address
+	 * or promiscuous mode.  We just need to check the multicast
+	 * filter or broadcast.
+	 */
+
+	/* If the packet is not multicast, we know it should be received. */
+	if (! net_ether_multicast(d->rx_packet))
+		return 0;
+
+	/*
+	 * Optimization -- if the guest has set all of the filter
+	 * bits, then we can skip additional checks.
+	 */
+	if (d->allmulti)
+		return 0;
+
+	/* Check for broadcast. */
+	if (net_ether_broadcast(d->rx_packet))
+		return 0;
+
+	/*
+	 * Check the multicast address filter.  We pass the address
+	 * through the little-endian Ethernet CRC generator.  The
+	 * high-order 6 bits are the index into the 64-bit filter.
+	 * The upper 2 bits select the 16-bit filter word, and the
+	 * remaining 4 select the bit in the word.
+	 */
+	uint32_t crc = net_ether_crc32_le(d->rx_packet, 6);
+	crc >>= 26;
+	if (d->ladrf[crc >> 4] & (1 << (crc & 0xf)))
+		return 0;
+
+	/* Not for us; drop the packet. */
+	free(d->rx_packet);
+	d->rx_packet = NULL;
+	d->rx_packet_len = 0;
+
+	return 1;
+}
+
+
 /*
  *  le_register_fix():
  */
@@ -481,9 +586,12 @@ static void le_register_fix(struct net *net, struct le_data *d)
 				break;
 
 			if (d->rx_packet == NULL &&
-			    net_ethernet_rx_avail(net, d))
-				net_ethernet_rx(net, d,
+			    net_ethernet_rx_avail(net, &d->nic)) {
+				net_ethernet_rx(net, &d->nic,
 				    &d->rx_packet, &d->rx_packet_len);
+				if (le_rx_drop_packet(net, d))
+					continue;
+			}
 		} while (d->rx_packet != NULL);
 	}
 
@@ -766,6 +874,8 @@ void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 	CHECK_ALLOCATION(d = (struct le_data *) malloc(sizeof(struct le_data)));
 	memset(d, 0, sizeof(struct le_data));
 
+	d->bigendian = machine->cpus[0]->byte_order == EMUL_BIG_ENDIAN;
+
 	INTERRUPT_CONNECT(irq_path, d->irq);
 
 	CHECK_ALLOCATION(d->sram = (unsigned char *) malloc(SRAM_SIZE));
@@ -813,6 +923,7 @@ void dev_le_init(struct machine *machine, struct memory *mem, uint64_t baseaddr,
 
 	machine_add_tickfunction(machine, dev_le_tick, d, LE_TICK_SHIFT);
 
-	net_add_nic(machine->emul->net, d, &d->rom[0]);
+	memcpy(d->nic.mac_address, &d->rom[0], sizeof(d->nic.mac_address));
+	net_add_nic(machine->emul->net, &d->nic);
 }
 
