@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2018  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2018-2021  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -29,7 +29,21 @@
  *
  *  Almost everything in here is just dummy code which returns nonsense,
  *  just enough to fake hardware well enough to get OpenBSD/luna88k to
- *  not stop early during bootup. It does not really work yet.
+ *  work to some basic degree.
+ *
+ *  Things that are implemented:
+ *	Interrupt controller (hopefully working with all 4 CPUs)
+ *	Time-of-day clock
+ *	Serial I/O (including Keyboard and Mouse)
+ *	Monochrome framebuffer
+ *	Lance ethernet
+ *
+ *  Things that are NOT implemented yet:
+ *	LUNA-88K2 specifics. (Some registers are at different addresses etc.)
+ *	SCSI
+ *	Parallel I/O
+ *	Front LCD display
+ *	Color framebuffer
  *
  *  TODO: Separate out these devices to their own files, so that they can
  *  potentially be reused for a luna68k mode if necessary.
@@ -55,13 +69,16 @@
 
 #define	TICK_STEPS_SHIFT	14
 
+// TODO: Actually make this configurable. Currently hardcoded to match OpenBSD/luna88k.
+#define	LUNA88K_PSEUDO_TIMER_HZ	100.0
 
 #define	LUNA88K_REGISTERS_BASE		0x3ffffff0UL
-#define	LUNA88K_REGISTERS_END		0xff000000UL
+#define	LUNA88K_REGISTERS_END		0xf0800000UL
 #define	LUNA88K_REGISTERS_LENGTH	(LUNA88K_REGISTERS_END - LUNA88K_REGISTERS_BASE)
 
 #define	MAX_CPUS	4
 
+#define	SIO_QUEUE_SIZE	256
 
 #define	BCD(x) ((((x) / 10) << 4) + ((x) % 10))
 
@@ -69,97 +86,163 @@
 struct luna88k_data {
 	struct vfb_data *fb;
 
-	struct interrupt cpu_irq;
-	bool		irqActive;
+	struct interrupt cpu_irq[MAX_CPUS];
+	int		irqActive[MAX_CPUS];
 	uint32_t	interrupt_enable[MAX_CPUS];
-	uint32_t	interrupt_status[MAX_CPUS];
-
+	uint32_t	interrupt_status;
 	uint32_t	software_interrupt_status[MAX_CPUS];
 
-	int		timer_tick_counter_bogus;
+	/*  Timer stuff  */
+	struct timer	*timer;
+	int		pending_timer_interrupts;
 	struct interrupt timer_irq;
 
+	/*  sio: Serial controller, two channels.  */
 	int		console_handle;
-	int		interrupt_delay;
-	struct interrupt irqX;
-
-	/*  Two channels.  */
+	struct interrupt sio_irq;
+	int		sio_tx_pending[2];
 	int		obio_sio_regno[2];
 	uint8_t		obio_sio_rr[2][8];
 	uint8_t		obio_sio_wr[2][8];
+	uint8_t		sio_queue[2][SIO_QUEUE_SIZE];
+	int		sio_queue_head[2];
+	int		sio_queue_tail[2];
+	int		mouse_enable;
+	int		mouse_x;
+	int		mouse_y;
+	int		mouse_buttons;
 
+	/*  ROM and RAM (used by the Ethernet interface)  */
 	uint32_t	fuse_rom[FUSE_ROM_SPACE / sizeof(uint32_t)];
 	uint8_t		nvram[NVRAM_SPACE];
+	uint8_t		tri_port_ram[TRI_PORT_RAM_SPACE];
 };
 
 
+static int anything_in_sio_queue(struct luna88k_data* d, int n)
+{
+	return d->sio_queue_head[n] != d->sio_queue_tail[n];
+}
+
+static int space_available_in_sio_queue(struct luna88k_data* d, int n)
+{
+	int diff = d->sio_queue_head[n] - d->sio_queue_tail[n];
+	if (diff <= 0)
+		diff += SIO_QUEUE_SIZE;
+
+	return diff;
+}
+
+static uint8_t get_from_sio_queue(struct luna88k_data* d, int n)
+{
+	uint8_t result = d->sio_queue[n][d->sio_queue_head[n]++];
+	if (d->sio_queue_head[n] >= SIO_QUEUE_SIZE)
+		d->sio_queue_head[n] = 0;
+
+	return result;
+}
+
+static void add_to_sio_queue(struct luna88k_data* d, int n, uint8_t c)
+{
+	d->sio_queue[n][d->sio_queue_tail[n]++] = c;
+	if (d->sio_queue_tail[n] >= SIO_QUEUE_SIZE)
+		d->sio_queue_tail[n] = 0;
+
+	if (d->sio_queue_tail[n] == d->sio_queue_head[n])
+		fatal("[ luna88k: add_to_sio_queue overrun; increase SIO_QUEUE_SIZE ]\n");
+}
+
 static void reassert_interrupts(struct luna88k_data *d)
 {
-	// printf("status = 0x%08x, enable = 0x%08x\n",
-	//	d->interrupt_status[0], d->interrupt_enable[0]);
+	int cpu;
 
-        if (d->interrupt_status[0] & d->interrupt_enable[0]) {
-		if (!d->irqActive)
-	                INTERRUPT_ASSERT(d->cpu_irq);
+	for (cpu = 0; cpu < MAX_CPUS; cpu++) {
+		// printf("cpu%i: status = 0x%08x, enable = 0x%08x\n",
+		//	cpu, d->interrupt_status, d->interrupt_enable[0]);
 
-		d->irqActive = true;
-	} else {
-		if (d->irqActive)
-	                INTERRUPT_DEASSERT(d->cpu_irq);
+		// General interrupt status is for the entire machine,
+		// but the Software interrupts (IPIs) are per CPU.
+		uint32_t status = d->interrupt_status;
+		uint32_t enable = d->interrupt_enable[cpu];
 
-		d->irqActive = false;
+		if (d->software_interrupt_status[cpu])
+			status |= (1 << 26);
+
+	        if (status & enable) {
+			if (!d->irqActive[cpu])
+		                INTERRUPT_ASSERT(d->cpu_irq[cpu]);
+
+			d->irqActive[cpu] = 1;
+		} else {
+			if (d->irqActive[cpu])
+		                INTERRUPT_DEASSERT(d->cpu_irq[cpu]);
+
+			d->irqActive[cpu] = 0;
+		}
 	}
+
+	// printf("--\n");
 }
 
 static void luna88k_interrupt_assert(struct interrupt *interrupt)
 {
         struct luna88k_data *d = (struct luna88k_data *) interrupt->extra;
-	d->interrupt_status[0] |= (1 << (interrupt->line + 25));
+	d->interrupt_status |= (1 << (interrupt->line + 25));
 	reassert_interrupts(d);
 }
 
 static void luna88k_interrupt_deassert(struct interrupt *interrupt)
 {
         struct luna88k_data *d = (struct luna88k_data *) interrupt->extra;
-	d->interrupt_status[0] &= ~(1 << (interrupt->line + 25));
+	d->interrupt_status &= ~(1 << (interrupt->line + 25));
 	reassert_interrupts(d);
 }
 
+static void reassert_timer_interrupt(struct luna88k_data* d)
+{
+	if (d->pending_timer_interrupts)
+		INTERRUPT_ASSERT(d->timer_irq);
+	else
+		INTERRUPT_DEASSERT(d->timer_irq);
+}
 
 static void reassert_serial_interrupt(struct luna88k_data* d)
 {
-	bool assertSerial = false;
+	int assertSerial = 0;
+	int port;
 
-	if (d->fb != NULL) {
-		/*  Workstation keyboard:  */
-		if ((d->obio_sio_wr[1][SCC_WR1] & SCC_WR1_RXI_ALL_CHAR) ||
-		    (d->obio_sio_wr[1][SCC_WR1] & SCC_WR1_RXI_FIRST_CHAR)) {
-			if (console_charavail(d->console_handle))
-				assertSerial = true;
-		}
-	} else {
-		/*  Serial:  */
-		if ((d->obio_sio_wr[0][SCC_WR1] & SCC_WR1_RXI_ALL_CHAR) ||
-		    (d->obio_sio_wr[0][SCC_WR1] & SCC_WR1_RXI_FIRST_CHAR)) {
-			if (console_charavail(d->console_handle))
-				assertSerial = true;
+	for (port = 0; port <= 1; port++) {
+		if ((d->obio_sio_wr[port][SCC_WR1] & SCC_WR1_RXI_ALL_CHAR) ||
+		    (d->obio_sio_wr[port][SCC_WR1] & SCC_WR1_RXI_FIRST_CHAR)) {
+			if (anything_in_sio_queue(d, port))
+				assertSerial = 1;
 		}
+
+		if (d->obio_sio_wr[port][SCC_WR1] & SCC_WR1_TX_IE &&
+		    d->sio_tx_pending[port])
+			assertSerial = 1;
 	}
 
-	if (d->obio_sio_wr[0][SCC_WR1] & SCC_WR1_TX_IE) {
-		assertSerial = true;
-	}
+	if (assertSerial)
+		INTERRUPT_ASSERT(d->sio_irq);
+	else
+		INTERRUPT_DEASSERT(d->sio_irq);
+}
 
-	if (d->interrupt_delay > 0) {
-		assertSerial = false;
-		d->interrupt_delay --;
-	}
 
-	if (assertSerial) {
-		INTERRUPT_ASSERT(d->irqX);
-		d->interrupt_delay = 130;
-	} else {
-		INTERRUPT_DEASSERT(d->irqX);
+/*
+ *  luna88k_timer_tick():
+ *
+ *  This function is called LUNA88K_PSEUDO_TIMER_HZ times per real-world second.
+ */
+static void luna88k_timer_tick(struct timer *t, void *extra)
+{
+	struct luna88k_data* d = (struct luna88k_data*) extra;
+	d->pending_timer_interrupts ++;
+	
+	if (d->pending_timer_interrupts > (int)(LUNA88K_PSEUDO_TIMER_HZ/2)) {
+		d->pending_timer_interrupts = 0;
+		debug("[ luna88k_timer_tick: tick lost... restarting timer ticks. ]\n");
 	}
 }
 
@@ -168,14 +251,179 @@ DEVICE_TICK(luna88k)
 {
 	struct luna88k_data *d = (struct luna88k_data *) extra;
 
-	/*  TODO: Correct timing.  */
-	if (d->timer_tick_counter_bogus < 3) {
-		if (++d->timer_tick_counter_bogus >= 3)
-			INTERRUPT_ASSERT(d->timer_irq);
+	/*  Serial console:  */
+	if (d->fb == NULL) {
+		 while (space_available_in_sio_queue(d, 0) > 2 &&
+		    console_charavail(d->console_handle)) {
+			int c = console_readchar(d->console_handle);
+			add_to_sio_queue(d, 0, c);
+		}
+	}
+
+	/*  Keyboard and mouse:  */
+	if (d->fb != NULL) {
+		while (space_available_in_sio_queue(d, 1) > 7 &&
+		    console_charavail(d->console_handle)) {
+			int c = console_readchar(d->console_handle);
+			int shifted = 0;
+			int controlled = 0;
+			int sc = 0;
+
+			if (c >= 'A' && c <= 'Z') {
+				shifted = 1;
+				c += 'a' - 'A';
+			}
+			
+			if (c >= 1 && c <= 26) {
+				controlled = 1;
+				c += 'a' - 1;
+			}
+			
+			switch (c) {
+			case 'a':	sc = 0x42;	break;
+			case 'b':	sc = 0x56;	break;
+			case 'c':	sc = 0x54;	break;
+			case 'd':	sc = 0x44;	break;
+			case 'e':	sc = 0x34;	break;
+			case 'f':	sc = 0x45;	break;
+			case 'g':	sc = 0x46;	break;
+			case 'h':	sc = 0x47;	break;
+			case 'i':	sc = 0x39;	break;
+			case 'j':	sc = 0x48;	break;
+			case 'k':	sc = 0x49;	break;
+			case 'l':	sc = 0x4a;	break;
+			case 'm':	sc = 0x58;	break;
+			case 'n':	sc = 0x57;	break;
+			case 'o':	sc = 0x3a;	break;
+			case 'p':	sc = 0x3b;	break;
+			case 'q':	sc = 0x32;	break;
+			case 'r':	sc = 0x35;	break;
+			case 's':	sc = 0x43;	break;
+			case 't':	sc = 0x36;	break;
+			case 'u':	sc = 0x38;	break;
+			case 'v':	sc = 0x55;	break;
+			case 'w':	sc = 0x33;	break;
+			case 'x':	sc = 0x53;	break;
+			case 'y':	sc = 0x37;	break;
+			case 'z':	sc = 0x52;	break;
+
+			case '1':	sc = 0x22;	break;
+			case '2':	sc = 0x23;	break;
+			case '3':	sc = 0x24;	break;
+			case '4':	sc = 0x25;	break;
+			case '5':	sc = 0x26;	break;
+			case '6':	sc = 0x27;	break;
+			case '7':	sc = 0x28;	break;
+			case '8':	sc = 0x29;	break;
+			case '9':	sc = 0x2a;	break;
+			case '0':	sc = 0x2b;	break;
+			case '-':	sc = 0x2c;	break;
+			case '^':	sc = 0x2d;	break;
+			case '\\':	sc = 0x2e;	break;
+
+			case '!':	sc = 0x22; shifted = 1;	break;
+			case '"':	sc = 0x23; shifted = 1;	break;
+			case '#':	sc = 0x24; shifted = 1;	break;
+			case '$':	sc = 0x25; shifted = 1;	break;
+			case '%':	sc = 0x26; shifted = 1;	break;
+			case '&':	sc = 0x27; shifted = 1;	break;
+			case '\'':	sc = 0x28; shifted = 1;	break;
+			case '(':	sc = 0x29; shifted = 1;	break;
+			case ')':	sc = 0x2a; shifted = 1;	break;
+			case '=':	sc = 0x2c; shifted = 1;	break;
+			case '~':	sc = 0x2d; shifted = 1;	break;
+			case '|':	sc = 0x2e; shifted = 1;	break;
+
+			case '@':	sc = 0x3c;	break;
+			case '[':	sc = 0x3d;	break;
+			case ';':	sc = 0x4b;	break;
+			case ':':	sc = 0x4c;	break;
+			case ']':	sc = 0x4d;	break;
+			case ',':	sc = 0x59;	break;
+			case '.':	sc = 0x5a;	break;
+			case '/':	sc = 0x5b;	break;
+			case '_':	sc = 0x5c;	break;
+
+			case '`':	sc = 0x3c; shifted = 1;	break;
+			case '{':	sc = 0x3d; shifted = 1;	break;
+			case '+':	sc = 0x4b; shifted = 1;	break;
+			case '*':	sc = 0x4c; shifted = 1;	break;
+			case '}':	sc = 0x4d; shifted = 1;	break;
+			case '<':	sc = 0x59; shifted = 1;	break;
+			case '>':	sc = 0x5a; shifted = 1;	break;
+			case '?':	sc = 0x5b; shifted = 1;	break;
+
+			case '\t':	sc = 0x09;	break;
+			case '\e':	sc = 0x10;	break;
+			case '\b':	sc = 0x11;	break;
+			case '\r':	sc = 0x12;	break;
+			case ' ':	sc = 0x14;	break;
+			case '\x7f':	sc = 0x15;	break;
+			}
+
+			if (sc > 0) {
+				if (sc <= 0x15)
+					controlled = 0;
+
+				if (shifted)
+					add_to_sio_queue(d, 1, 0x0d);
+				if (controlled)
+					add_to_sio_queue(d, 1, 0x0a);
+			
+				add_to_sio_queue(d, 1, sc);
+				add_to_sio_queue(d, 1, sc | 0x80);
+
+				if (controlled)
+					add_to_sio_queue(d, 1, 0x0a | 0x80);
+				if (shifted)
+					add_to_sio_queue(d, 1, 0x0d | 0x80);
+			}
+		}
+
+		if (space_available_in_sio_queue(d, 1) > 4 && d->mouse_enable) {
+			int mouse_x, mouse_y, mouse_buttons, mouse_fb_nr;
+			console_getmouse(&mouse_x, &mouse_y, &mouse_buttons, &mouse_fb_nr);
+
+			int xdelta = mouse_x - d->mouse_x;
+			int ydelta = d->mouse_y - mouse_y;	// note: inverted
+
+			const int m = 100;
+
+			if (xdelta > m)
+				xdelta = m;
+			if (xdelta < -m)
+				xdelta = -m;
+			if (ydelta > m)
+				ydelta = m;
+			if (ydelta < -m)
+				ydelta = -m;
+
+			/*  Only send update if there is an actual diff.  */
+			if (xdelta != 0 || ydelta != 0 || d->mouse_buttons != mouse_buttons) {
+				d->mouse_x = mouse_x;
+				d->mouse_y = mouse_y;
+				d->mouse_buttons = mouse_buttons;
+
+				// 3-byte protocol according to
+				// OpenBSD/luna88k's lunaws.c.
+				uint8_t b1 = 0x80;
+				int8_t b2 = xdelta;
+				int8_t b3 = ydelta;
+
+				// Buttons are L=4, M=2, R=1, but off means
+				// button down, on means button up! Weird...
+				b1 |= ((~mouse_buttons) & 7);
+
+				//printf("x=%i y=%i b1=%02x %02x %02x\n", xdelta, ydelta, b1,b2,b3);
+				add_to_sio_queue(d, 1, b1);
+				add_to_sio_queue(d, 1, b2);
+				add_to_sio_queue(d, 1, b3);
+			}
+		}
 	}
 
-	/*  Serial:  */
 	reassert_serial_interrupt(d);
+	reassert_timer_interrupt(d);
 }
 
 
@@ -290,7 +538,14 @@ DEVICE_ACCESS(luna88k)
 	}
 
 	if (addr >= TRI_PORT_RAM && addr < TRI_PORT_RAM + TRI_PORT_RAM_SPACE) {
-		/*  Ignore for now.  */
+		size_t offset = addr - TRI_PORT_RAM;
+
+		cpu->memory_rw(cpu, cpu->mem, LANCE_ADDR - 0x100000 + offset,
+			data, len, writeflag, NO_EXCEPTIONS | PHYSICAL);
+// if(len==2)
+// printf("OFFSET 0x%06x WRITE=%i 0x%02x%02x\n", (int)offset, (int)writeflag, data[0],data[1]);
+// else
+// printf("OFFSET 0x%06x WRITE=%i 0x%02x\n", (int)offset, (int)writeflag, data[0]);
 		return 1;
 	}
 
@@ -306,6 +561,8 @@ DEVICE_ACCESS(luna88k)
 		break;
 
 	case OBIO_CAL_CTL:	/*  calendar control register  */
+		// TODO: Freeze bit etc.
+		// Perhaps same as dev_mk48txx.cc?
 		break;
 	case OBIO_CAL_SEC:
 		timet = time(NULL); tmp = gmtime(&timet);
@@ -333,7 +590,8 @@ DEVICE_ACCESS(luna88k)
 		break;
 	case OBIO_CAL_YEAR:
 		timet = time(NULL); tmp = gmtime(&timet);
-		odata = BCD(tmp->tm_year - 2000) << 24;  // ?
+		// TODO: 1970 for LUNA88K (MK), 1990 for LUNA88K2 (DS)
+		odata = BCD((tmp->tm_year + 1900) - 1970) << 24;
 		break;
 
 	case OBIO_PIO0A:	/*  0x49000000: PIO-0 port A  */
@@ -357,6 +615,7 @@ DEVICE_ACCESS(luna88k)
 
 	case OBIO_PIO1A:	/*  0x4d000000: PIO-0 port A  */
 	case OBIO_PIO1B:	/*  0x4d000004: PIO-0 port B  */
+	case OBIO_PIO1C:	/*  0x4d000008: PIO-0 port C  */
 	case OBIO_PIO1:		/*  0x4d00000C: PIO-0 control  */
 		/*  Ignore for now. (?)  */
 		break;
@@ -372,14 +631,21 @@ DEVICE_ACCESS(luna88k)
 
 			/*  Similar to dev_scc.cc ?  */
 			if (writeflag == MEM_WRITE) {
+				int old_tx_enable = d->obio_sio_wr[sio_devnr][SCC_WR1] & SCC_WR1_TX_IE;
 				if (d->obio_sio_regno[sio_devnr] == 0) {
 					int regnr = idata & 7;
+					int cmd = idata & ~7;
 
-					d->obio_sio_regno[sio_devnr] = regnr;
-
-					// printf("[ sio: setting regno for next operation to 0x%02x ]\n", (int)regnr);
+					// printf("[ sio: cmd=0x%02x, then setting regno for next operation to 0x%02x ]\n", cmd, regnr);
 
 					/*  High bits are command.  */
+					switch (cmd) {
+					case SCC_RESET_TX_IP:
+						d->sio_tx_pending[sio_devnr] = 0;
+						break;
+					}
+
+					d->obio_sio_regno[sio_devnr] = regnr;
 				} else {
 					int regnr = d->obio_sio_regno[sio_devnr] & 7;
 					d->obio_sio_wr[sio_devnr][regnr] = idata;
@@ -387,15 +653,16 @@ DEVICE_ACCESS(luna88k)
 					// printf("[ sio: setting reg 0x%02x = 0x%02x ]\n", d->obio_sio_regno[sio_devnr], (int)idata);
 
 					d->obio_sio_regno[sio_devnr] = 0;
-
-					reassert_serial_interrupt(d);
 				}
+
+				if (d->obio_sio_wr[sio_devnr][SCC_WR1] & SCC_WR1_TX_IE && !old_tx_enable)
+					d->sio_tx_pending[sio_devnr] = 1;
+
+				reassert_serial_interrupt(d);
 			} else {
 				d->obio_sio_rr[sio_devnr][SCC_RR0] = SCC_RR0_TX_EMPTY | SCC_RR0_DCD | SCC_RR0_CTS;
 
-				if (console_charavail(d->console_handle) &&
-				      ( (d->fb == NULL && sio_devnr == 0) ||
-					(d->fb != NULL && sio_devnr == 1)) )
+				if (anything_in_sio_queue(d, sio_devnr))
 					d->obio_sio_rr[sio_devnr][SCC_RR0] |= SCC_RR0_RX_AVAIL;
 
 				d->obio_sio_rr[sio_devnr][SCC_RR1] = SCC_RR1_ALL_SENT;
@@ -408,28 +675,60 @@ DEVICE_ACCESS(luna88k)
 		} else {
 			/*  data  */
 			if (writeflag == MEM_WRITE) {
+				// printf("[ sio: writing data 0x%02x to port %i ]\n", (int)idata, sio_devnr);
 				if (sio_devnr == 0) {
-					console_putchar(cpu->machine->main_console_handle, idata);
-				} else
-					fatal("[ luna88k sio dev1 write data: TODO ]\n");
-			} else {
-				if (sio_devnr == 0) {
-					if (d->fb == NULL && console_charavail(d->console_handle)) {
-						odata = console_readchar(d->console_handle);
-					}
+					console_putchar(d->console_handle, idata);
 				} else {
-					fatal("[ luna88k sio dev1 read data: TODO ]\n");
+					/*  Mouse and Keyboard.  */
+					/*  These are according to OpenBSD/luna88k's lunaws.c  */
+					switch (idata) {
+					case 0x00:
+						/*  "kana LED off".  TODO  */
+						break;
+					case 0x01:
+						/*  "caps LED off".  TODO  */
+						break;
+					case 0x10:
+						/*  "kana LED on".  TODO  */
+						break;
+					case 0x11:
+						/*  "caps LED on".  TODO  */
+						break;
+					case 0x20:
+						d->mouse_enable = 0;
+						break;
+					case 0x60:
+						d->mouse_enable = 1;
+						break;
+					default:
+						fatal("[ luna88k: sio write to dev 1 (keyboard/mouse): 0x%02x ]\n", (int)idata);
+					}
+				}
+
+				d->sio_tx_pending[sio_devnr] = 1;
+			} else {
+				if (anything_in_sio_queue(d, sio_devnr)) {
+					odata = get_from_sio_queue(d, sio_devnr);
+				} else {
+					odata = 0;
 				}
 			}
 
-			INTERRUPT_DEASSERT(d->irqX);
+			reassert_serial_interrupt(d);
 		}
 
 		break;
 
 	case OBIO_CLOCK0:	/*  0x63000000: Clock ack?  */
-		d->timer_tick_counter_bogus = 0;
-		INTERRUPT_DEASSERT(d->timer_irq);
+	case OBIO_CLOCK1:	/*  0x63000004: Clock ack?  */
+	case OBIO_CLOCK2:	/*  0x63000008: Clock ack?  */
+	case OBIO_CLOCK3:	/*  0x6300000c: Clock ack?  */
+		cpunr = (addr - OBIO_CLOCK0) / 4;
+		// TODO: Pending counter per cpu?
+		if (d->pending_timer_interrupts > 0)
+			d->pending_timer_interrupts --;
+
+		reassert_timer_interrupt(d);
 		break;
 
 	case INT_ST_MASK0:	/*  0x65000000: Interrupt status CPU 0.  */
@@ -454,12 +753,13 @@ DEVICE_ACCESS(luna88k)
 			reassert_interrupts(d);
 		} else {
 			uint32_t currentMask = d->interrupt_enable[cpunr];
+			uint32_t status = d->interrupt_status & currentMask;
 			int highestCurrentStatus = 0;
 			odata = currentMask >> 8;
 			
 			for (int i = 1; i <= 6; ++i) {
 				int m = 1 << (25 + i);
-				if (d->interrupt_status[cpunr] & m)
+				if (status & m)
 					highestCurrentStatus = i;
 			}
 
@@ -474,15 +774,16 @@ DEVICE_ACCESS(luna88k)
 	case SOFT_INT2:		/*  0x69000008: Software Interrupt status CPU 2.  */
 	case SOFT_INT3:		/*  0x6900000c: Software Interrupt status CPU 3.  */
 		cpunr = (addr - SOFT_INT0) / 4;
-		odata = d->software_interrupt_status[cpunr];
-
-		// Reading status clears it.
-		d->software_interrupt_status[cpunr] = 0;
 
 		if (writeflag == MEM_WRITE) {
-			fatal("TODO: luna88k write to software interrupts\n");
-			exit(1);
+			d->software_interrupt_status[cpunr] = idata;
+		} else {
+			// Reading status clears it.
+			odata = d->software_interrupt_status[cpunr];
+			d->software_interrupt_status[cpunr] = 0;
 		}
+
+		reassert_interrupts(d);
 		break;
 
 	case RESET_CPU_ALL:	/*  0x6d000010: Reset all CPUs  */
@@ -546,11 +847,6 @@ DEVICE_ACCESS(luna88k)
 		odata = 0xffffffff;
 		break;
 
-	case 0xf1000000:	/*  Lance Ethernet. TODO.  */
-	case 0xf1000004:
-	case 0xf1000008:
-		break;
-
 	default:fatal("[ luna88k: unimplemented %s address 0x%x",
 		    writeflag == MEM_WRITE? "write to" : "read from",
 		    (int) addr);
@@ -572,30 +868,31 @@ void add_cmmu_for_cpu(struct devinit* devinit, int cpunr, uint32_t iaddr, uint32
 	char tmpstr[300];
 	struct m8820x_cmmu *cmmu;
 
+	if (cpunr >= devinit->machine->ncpus)
+		return;
+
 	/*  Instruction CMMU:  */
 	CHECK_ALLOCATION(cmmu = (struct m8820x_cmmu *) malloc(sizeof(struct m8820x_cmmu)));
 	memset(cmmu, 0, sizeof(struct m8820x_cmmu));
 
-	if (cpunr < devinit->machine->ncpus)
-		devinit->machine->cpus[cpunr]->cd.m88k.cmmu[0] = cmmu;
+	devinit->machine->cpus[cpunr]->cd.m88k.cmmu[0] = cmmu;
 
 	/*  This is a 88200, revision 9:  */
 	cmmu->reg[CMMU_IDR] = (M88200_ID << 21) | (9 << 16);
-	snprintf(tmpstr, sizeof(tmpstr), "m8820x addr=0x%x addr2=0", iaddr);
+	snprintf(tmpstr, sizeof(tmpstr), "m8820x addr=0x%x addr2=%i", iaddr, 2 * cpunr);
 	device_add(devinit->machine, tmpstr);
 
 	/*  ... and data CMMU:  */
 	CHECK_ALLOCATION(cmmu = (struct m8820x_cmmu *) malloc(sizeof(struct m8820x_cmmu)));
 	memset(cmmu, 0, sizeof(struct m8820x_cmmu));
 
-	if (cpunr < devinit->machine->ncpus)
-		devinit->machine->cpus[cpunr]->cd.m88k.cmmu[1] = cmmu;
+	devinit->machine->cpus[cpunr]->cd.m88k.cmmu[1] = cmmu;
 
 	/*  This is also a 88200, revision 9:  */
 	cmmu->reg[CMMU_IDR] = (M88200_ID << 21) | (9 << 16);
 	cmmu->batc[8] = BATC8;
 	cmmu->batc[9] = BATC9;
-	snprintf(tmpstr, sizeof(tmpstr), "m8820x addr=0x%x addr2=1", daddr);
+	snprintf(tmpstr, sizeof(tmpstr), "m8820x addr=0x%x addr2=%i", daddr, 1 + 2 * cpunr);
 	device_add(devinit->machine, tmpstr);
 }
 
@@ -617,7 +914,16 @@ DEVINIT(luna88k)
 	 *  Connect to the CPU's interrupt pin, and register
 	 *  6 hardware interrupts:
 	 */
-	INTERRUPT_CONNECT(devinit->interrupt_path, d->cpu_irq);
+	INTERRUPT_CONNECT(devinit->interrupt_path, d->cpu_irq[0]);
+
+	// HACK for trying out CPUs 1, 2 and 3. TODO: Base on devinit->interrupt_path instead.
+	if (devinit->machine->ncpus >= 2)
+		INTERRUPT_CONNECT("machine[0].cpu[1]", d->cpu_irq[1]);
+	if (devinit->machine->ncpus >= 3)
+		INTERRUPT_CONNECT("machine[0].cpu[2]", d->cpu_irq[2]);
+	if (devinit->machine->ncpus >= 4)
+		INTERRUPT_CONNECT("machine[0].cpu[3]", d->cpu_irq[3]);
+
         for (int i = 1; i <= 6; i++) {
                 struct interrupt templ;
                 snprintf(n, sizeof(n), "%s.luna88k.%i", devinit->interrupt_path, i);
@@ -637,24 +943,30 @@ DEVINIT(luna88k)
 	/*  Timer.  */
 	snprintf(n, sizeof(n), "%s.luna88k.6", devinit->interrupt_path);
 	INTERRUPT_CONNECT(n, d->timer_irq);
-	machine_add_tickfunction(devinit->machine,
-	    dev_luna88k_tick, d, TICK_STEPS_SHIFT);
+	d->timer = timer_add(LUNA88K_PSEUDO_TIMER_HZ, luna88k_timer_tick, d);
 
-	/*  IRQ 5,4,3 (?): "autovec" according to OpenBSD  */
+	machine_add_tickfunction(devinit->machine, dev_luna88k_tick, d, TICK_STEPS_SHIFT);
+
+	/*
+	 *  IRQ 5,4,3 (?): "autovec" according to OpenBSD
+	 *
+	 *  5 = sio0 (serial controller)
+	 *  4 = le0  (lance ethernet)
+	 *  3 = spc0 (SCSI)
+	 */
 	snprintf(n, sizeof(n), "%s.luna88k.5", devinit->interrupt_path);
-	INTERRUPT_CONNECT(n, d->irqX);
+	INTERRUPT_CONNECT(n, d->sio_irq);
 
 	d->console_handle = console_start_slave(devinit->machine, "SIO", 1);
+	devinit->machine->main_console_handle = d->console_handle;
 
-	if (devinit->machine->x11_md.in_use)
-	{
+	if (devinit->machine->x11_md.in_use) {
 		d->fb = dev_fb_init(devinit->machine, devinit->machine->memory,
 			0x100000000ULL + BMAP_BMAP0, VFB_GENERIC,
 			1280, 1024, 2048, 1024, 1, "LUNA 88K");
 	}
 
-	if (devinit->machine->ncpus > 4)
-	{
+	if (devinit->machine->ncpus > 4) {
 		printf("LUNA 88K can't have more than 4 CPUs.\n");
 		exit(1);
 	}
@@ -664,6 +976,22 @@ DEVINIT(luna88k)
 	add_cmmu_for_cpu(devinit, 2, CMMU_I2, CMMU_D2);
 	add_cmmu_for_cpu(devinit, 3, CMMU_I3, CMMU_D3);
 
+	// The address is a hack because dev_le assumes that the data and register ports
+	// are at offset 0x100000 and "ram" (used for packets) is at 0x000000.
+	snprintf(n, sizeof(n), "%s.luna88k.4", devinit->interrupt_path);
+	dev_le_init(devinit->machine, devinit->machine->memory, LANCE_ADDR - 0x100000, 0, 0, n, DEV_LE_LENGTH);
+
+	// TODO: actual address from the le device!
+	// According to OpenBSD's if_le.c, the format for
+	// real Luna machines is 00000Axxxxxx.
+	const char *enaddr = "ENADDR=00000A102030";
+	size_t i = 0;
+	while (enaddr[i]) {
+		d->fuse_rom[i*2+0] = (enaddr[i] & 0xf0) << 24;
+		d->fuse_rom[i*2+1] = (enaddr[i] & 0x0f) << 28;
+		++i;
+	}
+
 	return 1;
 }
 
