@@ -2,7 +2,7 @@
 #define	I960_CPUCOMPONENT_H
 
 /*
- *  Copyright (C) 2018-2019  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2018-2020  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -36,30 +36,7 @@
 
 #define	N_I960_REGS		32
 #define	N_I960_SFRS		32
-
-
-/*
-Register conventions according to
-https://people.cs.clemson.edu/~mark/subroutines/i960.html
-*/
-
-static const char* i960_regnames[N_I960_REGS] = {
-	"pfp",		// r0 = previous frame pointer
-	"sp",		// r1 = stack pointer
-	"rip",		// r2 = return instruction pointer
-	"r3", "r4", "r5", "r6", "r7",
-	"r8", "r9", "r10", "r11", "r12",
-	"r13", "r14", "r15",
-
-	"g0", "g1", "g2", "g3",	// parameters 0-3; return words 0-3
-	"g4", "g5", "g6", "g7", // parameters 4-7; temporaries
-	"g8", "g9", "g10", "g11", "g12",	// preserved accross call
-	"g13",		// structure return pointer
-	"g14",		// argument block pointer; leaf return address (HW)
-	"fp" 		// g15 = frame pointer (16-byte aligned HW)
-};
-
-#define	I960_G0		16	// offset to first parameter register
+#define	I960_G0			16	// offset to first parameter register
 
 
 /***********************************************************************/
