@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2008-2019  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2008-2021  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -33,9 +33,9 @@
  *
  *  RAMDISK kernel used for experiments:
  *
- *	https://ftp.eu.openbsd.org/pub/OpenBSD/6.2/luna88k/bsd.rd
+ *	https://ftp.eu.openbsd.org/pub/OpenBSD/6.8/luna88k/bsd.rd
  *
- *  Launch with   gxemul -e luna-88k2 -Ttv bsd.rd
+ *  Launch with   gxemul -e luna-88k bsd.rd
  */
 
 #include <stdio.h>
@@ -55,6 +55,7 @@
 MACHINE_SETUP(luna88k)
 {
 	const char* luna88k2_fuse_string = "MNAME=LUNA88K+";
+	size_t i;
 
 	device_add(machine, "luna88k");
 
@@ -77,7 +78,7 @@ MACHINE_SETUP(luna88k)
 		    where h is first 32-bit word, l is second.
 		*/
 
-		for (size_t i = 0; i < strlen(luna88k2_fuse_string); ++i) {
+		for (i = 0; i < strlen(luna88k2_fuse_string); ++i) {
 			uint32_t h = luna88k2_fuse_string[i] & 0xf0;
 			uint32_t l = luna88k2_fuse_string[i] & 0x0f;
 			store_32bit_word(cpu, FUSE_ROM_ADDR + i * 8 + 0, h << 24);
@@ -96,6 +97,14 @@ MACHINE_SETUP(luna88k)
 		exit(1);
 	}
 
+	// Unpause all CPUs. (Normally, the emulator only starts with
+	// one CPU running, and it is up to the OS to enable the rest,
+	// but on real LUNA-88K machines, all CPUs are started to run
+	// the OS kernel code simultaneously and it is up to the OS
+	// kernel's initial code to sort things out.)
+	for (i = 0; i < (size_t)machine->ncpus; ++i)
+		machine->cpus[i]->running = 1;
+
 	if (!machine->prom_emulation)
 		return;
 
@@ -111,7 +120,8 @@ MACHINE_DEFAULT_CPU(luna88k)
 
 MACHINE_DEFAULT_RAM(luna88k)
 {
-	// Two OpenBSD dmesgs found on the Internet for a LUNA-88K2 showed 112 MB of real mem.
+	// Two OpenBSD dmesgs found on the Internet for a LUNA-88K2 showed
+	// 112 MB of real mem. (I don't know what the max was for a LUNA-88K.)
 	machine->physical_ram_in_mb = 112;
 }
 
