@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2019  Anders Gavare.  All rights reserved.
+ *  Copyright (C) 2007-2021  Anders Gavare.  All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions are met:
@@ -26,7 +26,9 @@
  */
 
 #include <signal.h>
+#include <stdio.h>
 #include <unistd.h>
+#include <cstdio>
 #include <iostream>
 
 #include "misc.h"
